const serializer = require('../helpers/index').serializer
const https = require('https')
const crypto = require('crypto')

/**
 * Base methods for communication with Open API server.
 */
class BaseOpenAPIClient {
  constructor () {
    // do interface publishing to clients (browser, plugin, etc.)
    this.isPublicInterface = true

    this.resultField = 'PxgRetVal'

    // carry context for usage in promise callback
    this.getBody = serializer.getBody.bind(this)
    this.getResult = serializer.getResult.bind(this)
    this.serialize = serializer.serializeKscRequest.bind(this)
    this.deserialize = serializer.deserializeKscRequest.bind(this)

    this.cache = {}
  }

  /**
   * Send http request using FIFO queue
   * @param {string} command - backend action name
   * @param {string} httpMethod - http request method
   * @param {*} data - request body data
   * @param {object} headers - custom request headers
   * @param {Connection} connection - current tcp socket connection
   * @return {Promise}
   */
  baseRequest ({
    raw,
    command,
    httpMethod,
    data,
    headers,
    connection,
    tranformRequestBodyLogs,
    tranformResponseBodyLogs
  } = {}) {
    let self = this
    return new Promise(function (resolve, reject) {
      if (connection.isBusy) {
        connection.queue.push(request)
      } else {
        request()
      }

      function request () {
        connection.isBusy = true
        self
          .request({
            raw,
            command,
            httpMethod,
            data,
            headers,
            connection,
            tranformRequestBodyLogs,
            tranformResponseBodyLogs
          }, resolve, reject)
          .then(onResponse, onResponse)
          .catch(onResponse)

        function onResponse () {
          connection.isBusy = false
          if (connection.queue.length) {
            connection.queue.shift()()
          }
        }
      }
    })
  }

  /**
   * Send http request
   * @param {string} command - backend action name
   * @param {string} httpMethod - http request method
   * @param {*} data - request body data
   * @param {object} headers - custom request headers
   * @param {Connection} connection - current tcp socket connection
   * @param {function} resolve - resolve promise function
   * @param {function} reject - reject promise function
   * @return {Promise.<TResult>}
   */
  request (
    {
      raw = false,
      command,
      httpMethod = 'POST',
      data = {},
      headers = {},
      connection,
      tranformRequestBodyLogs,
      tranformResponseBodyLogs
    } = {},
    resolve = function () {},
    reject = function () {}
  ) {
    return new Promise(function (resolve, reject) {
      let body = JSON.stringify(data)
        .replace(/("type":"long","value":)"(\d+)"/g, '$1' + '$2')
        .replace(/"long-structure(\d+)"/g, '{"type":"long","value":$1}')
        .replace(/"long(\d+)"/g, '$1')
      let options = {
        host: connection.config.openAPIHost,
        port: connection.config.openAPIPort,
        method: httpMethod,
        path: raw
          ? command
          : `/api/${connection.config.openAPIVersion}/${command}`,
        headers: headers || {},
        createConnection: function () {
          return connection.socket
        }
      }
      options.headers['Content-Type'] = 'application/json'
      options.headers['Content-Length'] = Buffer.byteLength(body)
      options.headers['Connection'] = 'keep-alive'

      let bodyToLog = body
      if (typeof tranformRequestBodyLogs === 'function') {
        bodyToLog = JSON.stringify(tranformRequestBodyLogs(JSON.parse(body)))
      }
      console.log(`Request ${options.path} is sent.`)
      console.log('Request body:', bodyToLog)

      let request = https.request(options, function (response) {
        console.log(`Request ${options.path} is finished with status ${response.statusCode}.`)

        let responseBody = raw ? [] : ''

        response.on('data', function (chunk) {
          if (raw) {
            responseBody.push(chunk)
          } else {
            responseBody += chunk
          }
          console.log('Get body chunk')
        })

        response.on('end', function () {
          if (raw) {
            return resolve({
              response,
              body: Buffer.concat(responseBody)
            })
          }

          let body
          let stringifiedChunk = responseBody
            .toString()
            .replace(/"type":"long","value":\d+/g, (match, number, string) => {
              const arr = match.split(':')
              const l = arr.length
              arr[l - 1] = `"${arr[l - 1]}"`
              return arr.join(':')
            })
          try {
            body = JSON.parse(stringifiedChunk)
          } catch (err) {
            body = stringifiedChunk
          }
          let bodyToLog = require('util').inspect(body, false, null)
          if (typeof tranformResponseBodyLogs === 'function') {
            bodyToLog = tranformResponseBodyLogs(bodyToLog)
          }
          console.log('response body', bodyToLog)
          resolve({response, body})
        })
      })

      request.on('error', reject)

      request.write(body, () => console.log('data flushed'))

      request.end(() => console.log('stream finished'))
    }).then(resolve)
      .catch(error => {
        console.log(`problem with request: ${error.message}`)
        console.log(error)
        reject(error)
      })
  }

  putRequest ({
    httpMethod = 'PUT',
    path,
    chunkSize = 2048,
    fileString = '',
    connection,
    tranformRequestBodyLogs,
    tranformResponseBodyLogs
  } = {}) {
    return new Promise(function (resolve, reject) {
      const fileBuffer = Buffer.from(fileString, 'base64')
      fileString = fileBuffer.toString('binary')
      const sha256 = crypto.createHash('sha256').update(fileBuffer).digest('hex')
      let fileLength = fileString.length
      const headers = {
        'Expect': '100-continue',
        'Content-Length': `${fileLength}`,
        'Sha256': `${sha256}`
      }

      let options = {
        host: connection.config.openAPIHost,
        port: connection.config.openAPIPort,
        method: httpMethod,
        path,
        headers,
        createConnection: function () {
          return connection.socket
        }
      }

      let request = https.request(options, function (response) {
        console.log(`Request ${options.path} is finished with status ${response.statusCode}.`)
        resolve({response, body: {msg: `Completed with code ${response.statusCode}`, code: response.statusCode}})
      })

      request.on('continue', () => {
        let sentDataLength = 0
        while (fileLength > 0) {
          const chunk = fileString.substring(sentDataLength, sentDataLength = sentDataLength + chunkSize)
          fileLength = fileLength - chunk.length
          request.write(chunk, 'binary')
        }
        request.end(() => console.log('stream finished'))
      })

      request.on('error', reject)
    }).catch(error => {
      console.log(`problem with request: ${error.message}`)
      console.log(error)
    })
  }
}

module.exports = BaseOpenAPIClient
