const BaseOpenAPIClient = require('../base-open-api-client')
const className = 'HostTasks'

module.exports = class HostTasks extends BaseOpenAPIClient {
  ResetTasksIterator ({strSrvObjId, strProductName = '', strVersion = '', strComponentName = '', strInstanceId = '', strTaskName = ''}, connection) {
    return this.baseRequest({
      command: `${className}.ResetTasksIterator`,
      data: {strSrvObjId, strProductName, strVersion, strComponentName, strInstanceId, strTaskName},
      connection
    })
      .then(this.getBody)
  }

  AddTask ({strSrvObjId, taskData, skipSerialization}, connection) {
    const pData = skipSerialization ? taskData : this.serialize(taskData)
    return this.baseRequest({
      command: `${className}.AddTask`,
      data: { strSrvObjId, pData },
      connection
    })
      .then(this.getBody)
  }

  /**
   * Получение списка задач
   * @param {string} strSrvObjId - идентификатор серверного объекта, полученный из метода HostGroup.GetHostTasks
   * @param {Connection} connection
   * @return {Promise.<{result}>}
   * @constructor
   */
  GetNextTask ({strSrvObjId}, connection) {
    return this.baseRequest({
      command: `${className}.GetNextTask`,
      data: { strSrvObjId },
      connection
    })
      .then(body => this.deserialize(body, 'pTaskData'))
  }

  /**
   * Getting host local task data
   * @param {string} strSrvObjId - server object ID that got from HostGroup.GetHostTasks
   * @param {string} strTask - Tasks storage identifier of the task
   * @returns {params} task settings
   */
  async GetTaskData ({strSrvObjId, strTask}, connection) {
    return this.baseRequest({
      command: `${className}.GetTaskData`,
      data: {strSrvObjId, strTask},
      connection
    })
      .then(this.deserialize)
  }

  /**
   * Modifying host local task
   * @param {string} strSrvObjId - server object ID that got from HostGroup.GetHostTasks
   * @param {string} strTask - Tasks storage identifier of the task
   * @param {string} pData - task settings, see Task settings format
   * @param {any} connection
   * @returns
   */
  async UpdateTask ({strSrvObjId, strTask, pData}, connection) {
    return this.baseRequest({
      command: `${className}.UpdateTask`,
      data: {
        strSrvObjId,
        strTask,
        pData: this.serialize(pData)
      },
      connection
    })
      .then(this.getBody)
  }

  /**
   * Deleting local host task
   * @param {string} strSrvObjId - server object ID that got from HostGroup.GetHostTasks
   * @param {string} strTask - Tasks storage identifier of the task
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   * @constructor
   */
  async DeleteTask ({strSrvObjId, strTask}, connection) {
    return this.baseRequest({
      command: `${className}.DeleteTask`,
      data: {strSrvObjId, strTask},
      connection
    })
      .then(this.getBody)
  }

  /**
   * Получение списка задач для устройства
   * !важно: это не Open API method, только обёртка над OAPI методами
   * @param {string} strHostName - имя хоста (KLHST_WKS_HOSTNAME)
   * @param {Connection} connection
   * @return {Promise.<{result: {tasks: TResult}}>}
   */
  async GetHostTasks ({strHostName}, connection) {
    const strSrvObjId = await this._getStrSrvObjId(strHostName, connection)
    await this.ResetTasksIterator({strSrvObjId}, connection)
    const tasks = await getHostTaskRecursive.call(this, strSrvObjId, connection)
    return {
      result: {
        tasks
      }
    }

    /**
     * Рекурсивное получение задач устройства
     * Задачи получаются одна за другой
     * @param {string} strSrvObjId - серверный идентификатор для управления задачми устройства
     * @param {Connection} connection
     * @param {Array.Object} tasks - резщультирующий список полученных задач
     * @return {Promise.<*>}
     */
    async function getHostTaskRecursive (strSrvObjId, connection, tasks = []) {
      const {result: task} = await this.GetNextTask({ strSrvObjId }, connection)
      if (task) {
        tasks.push(task)
        return getHostTaskRecursive.call(this, strSrvObjId, connection, tasks)
      } else {
        return tasks
      }
    }
  }

  /**
   * Удаление локальных задач у устройства
   * !важно: это не Open API method, только обёртка над OAPI методами
   * @param {string} strHostName - уникальное имя хоста (KLHST_WKS_HOSTNAME)
   * @param {Array.Object} tasks - список задач для удаления
   * @param {Connection} connection
   * @return {Promise.<{result: null}>}
   * @constructor
   */
  async DeleteHostTasks ({strHostName, tasks}, connection) {
    const strSrvObjId = await this._getStrSrvObjId(strHostName, connection)
    let promises = []
    tasks.forEach(task => {
      const promise = this.DeleteTask({strSrvObjId, strTask: task.strTask}, connection)
      promises.push(promise)
    })
    await Promise.all(promises)
    return {
      result: null
    }
  }

  /**
   * Получение идентификатора серверного объекта
   * @param {string} strHostName - имя устройства
   * @return {Promise.<TResult>}
   * @private
   */
  _getStrSrvObjId (strHostName, connection) {
    const openAPIClient = require('../open-api-client')
    return openAPIClient
      .HostGroup
      .GetHostTasks({strHostName}, connection)
      .then(({result: strSrvObjId}) => strSrvObjId)
  }
}
