const BaseOpenAPIClient = require('../base-open-api-client')
const className = 'HstAccessControl'

/**
 * Provide user authentication methods.
 */
module.exports = class HstAccessControl extends BaseOpenAPIClient {
  /**
   * @description Add user role. A role can be added only at a main server.
   * @param {string} KLHST_ACL_ROLE_DN - role display name
   * @param {object} role_products - role access rights object
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   */
  AddRole ({ KLHST_ACL_ROLE_DN, role_products } = {}, connection) {
    return this.baseRequest({
      command: `${className}.AddRole`,
      data: {
        pRoleData: { KLHST_ACL_ROLE_DN, role_products }
      },
      connection
    })
      .then(this.getResult)
  }

  /**
   * @description Return parameters of a role.
   * @param {number} nId - id of a role
   * @param {Array<string>} pFieldsToReturn - array of strings with role attribute names to return
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   */
  GetRole ({ nId, pFieldsToReturn }, connection) {
    return this.baseRequest({
      command: `${className}.GetRole`,
      data: { nId, pFieldsToReturn },
      connection
    })
      .then(this.deserialize)
  }

  /**
   * @description Delete user role
   * @param {number} nId - id of a role
   * @param {boolean} bProtection - if true then it checks that the user does not reduce rights for himself
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   */
  DeleteRole ({ nId, bProtection = false }, connection) {
    return this.baseRequest({
      command: `${className}.DeleteRole`,
      data: { nId, bProtection },
      connection
    })
      .then(this.getResult)
  }

  /**
   * @description Update user role
   * @param {number} KLHST_ACL_ROLE_ID - id of a role
   * @param {string} KLHST_ACL_ROLE_NAME - guid name of a role
   * @param {string} displayName - human name of a role
   * @param {object} role_products - role access rights object
   * @param {boolean} bProtection - if true then it checks that the user does not reduce rights for himself
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   */
  UpdateRole ({ KLHST_ACL_ROLE_NAME, KLHST_ACL_ROLE_ID, displayName, role_products, pProtection = false }, connection) {
    let params = {
      nId: KLHST_ACL_ROLE_ID,
      pRoleData: {
        KLHST_ACL_ROLE_NAME,
        KLHST_ACL_ROLE_DN: displayName,
        role_products
      },
      pProtection
    }
    return this.baseRequest({
      command: `${className}.UpdateRole`,
      data: params,
      connection
    })
      .then(this.getResult)
  }

  /**
   * @description Find roles by filter string
   * @param {string} filters - filter string
   * @param {Array<string>} fields - array of strings with role attribute names to return
   * @param {Array<object>} sort - array of containers each of them containing two attributes
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   */
  FindRoles ({ filters, fields, sort }, connection) {
    var params = {
      strFilter: filters,
      pFieldsToReturn: fields,
      pFieldsToOrder: sort,
      lMaxLifeTime: 7200
    }
    return this.baseRequest({
      command: `${className}.FindRoles`,
      data: params,
      connection
    })
      .then(body => this.getResult(body, 'strAccessor'))
  }

  /**
   * @description Find roles by filter string (extended by iterator)
   * @param {string} filters - filter string
   * @param {Array<string>} fields - array of strings with role attribute names to return
   * @param {Array<object>} sort - array of containers each of them containing two attributes
   * @param {number} nStart - start position for counter
   * @param {number} nEnd - end position for counter
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   */
  FindRolesExtended ({ filters, fields, sort = [], nStart = 0, nEnd }, connection) {
    var params = {
      strFilter: filters,
      pFieldsToReturn: fields,
      pFieldsToOrder: sort,
      lMaxLifeTime: 7200
    }
    return this.baseRequest({
      command: `${className}.FindRoles`,
      data: params,
      connection
    })
    .then(body => this.getResult(body, 'strAccessor'))
    .then(iteratorContainer => {
      const strAccessor = iteratorContainer.result
      const OpenApiClient = require('../open-api-client')
      return OpenApiClient.ChunkAccessor.GetItemsCount({ strAccessor }, connection)
          .then(countContainer => {
            const count = countContainer.result
            const params = {
              strAccessor,
              nStart,
              nCount: nEnd || count
            }
            return OpenApiClient.ChunkAccessor.GetItemsChunk(params, connection)
          })
          .then(responseContainer => {
            return OpenApiClient.ChunkAccessor.Release({ strAccessor }, connection)
              .then(() => responseContainer)
          })
    })
  }

  /**
   * @description Returns ACL for the specified object.
   * @param {number} nObjId - object id
   * @param {number} nObjType - object type
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   */
  GetScObjectAcl ({ nObjId, nObjType = 1 }, connection) {
    var params = { nObjId, nObjType }
    return this.baseRequest({
      command: `${className}.GetScObjectAcl`,
      data: params,
      connection
    })
      .then(body => this.deserialize(body, 'pAclParams'))
  }

  /**
   * @description Sets ACL for the specified object.
   * @param {number} nObjId - object id
   * @param {number} nObjType - object type
   * @param {object} pAclParams - acl
   * @param {boolean} bCheckCurrentUserAce - if true then it checks that the user does not reduce rights for himself
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   */
  SetScObjectAcl ({ nObjId, nObjType = 1, pAclParams, bCheckCurrentUserAce = false }, connection) {
    var params = { nObjId, nObjType, pAclParams: this.serialize(pAclParams), bCheckCurrentUserAce }
    return this.baseRequest({
      command: `${className}.SetScObjectAcl`,
      data: params,
      connection
    })
      .then(body => this.deserialize(body, 'pAclParams'))
  }

  /**
   * @description Returns descriptions of visual view for access rights in KSC.
   * @param {string} wstrLangCode - IETF language tag (for example: en-us)
   * @param {number} nObjId - object id
   * @param {number} nObjType - object type
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   */
  GetVisualViewForAccessRights ({ wstrLangCode, nObjId, nObjType = 1 }, connection) {
    let params = { wstrLangCode, nObjId, nObjType }
    return this.baseRequest({
      command: `${className}.GetVisualViewForAccessRights`,
      data: params,
      connection
    })
      .then(body => this.deserialize(body, 'pViewParams'))
  }

  /**
   * @description Sets ACL for the specified object.
   * @param {number} lGroupId - id of the group
   * @param {number} dwAccessMask - access mask
   * @param {string} szwProduct - product
   * @param {string} szwVersion - version of product
   * @param {boolean} bInvert - if true then access mask is inverted
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   */
  GetAccessibleFuncAreas ({ lGroupId, dwAccessMask, szwProduct, szwVersion, bInvert }, connection) {
    let params = { lGroupId, dwAccessMask, szwProduct, szwVersion, bInvert }
    return this.baseRequest({
      command: `${className}.GetAccessibleFuncAreas`,
      data: params,
      connection
    })
      .then(body => this.deserialize(body, 'pFuncAreasArray'))
  }

  /**
   * @description Searches for trustees meeting specified criteria
   * @param {string} filters - filter string
   * @param {Array<string>} fields - array of strings with role attribute names to return
   * @param {Array<object>} sort - array of containers each of them containing two attributes
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   */
  FindTrustees ({ filters, fields, sort = [] }, connection) {
    var params = {
      strFilter: filters,
      pFieldsToReturn: fields,
      pFieldsToOrder: sort,
      lMaxLifeTime: 7200
    }
    return this.baseRequest({
      command: `${className}.FindTrustees`,
      data: params,
      connection
    })
      .then(body => this.getResult(body, 'strAccessor'))
  }

  /**
   * @description Searches for trustees meeting specified criteria (extended by iterator)
   * @param {string} filters - filter string
   * @param {Array<string>} fields - array of strings with role attribute names to return
   * @param {Array<object>} sort - array of containers each of them containing two attributes
   * @param {number} nStart - start position for counter
   * @param {number} nEnd - end position for counter
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   */
  FindTrusteesExtended ({ filters, fields, sort = [], nStart = 0, nEnd }, connection) {
    var params = {
      strFilter: filters,
      pFieldsToReturn: fields,
      pFieldsToOrder: sort,
      lMaxLifeTime: 7200
    }
    return this.baseRequest({
      command: `${className}.FindTrustees`,
      data: params,
      connection
    })
    .then(body => this.getResult(body, 'strAccessor'))
    .then(iteratorContainer => {
      const strAccessor = iteratorContainer.result
      const OpenApiClient = require('../open-api-client')
      return OpenApiClient.ChunkAccessor.GetItemsCount({ strAccessor }, connection)
          .then(countContainer => {
            const count = countContainer.result
            const params = {
              strAccessor: strAccessor,
              nStart: nStart,
              nCount: nEnd || count
            }
            return OpenApiClient.ChunkAccessor.GetItemsChunk(params, connection)
          })
          .then(responseContainer => {
            return OpenApiClient.ChunkAccessor.Release({ strAccessor }, connection)
              .then(() => responseContainer)
          })
    })
  }

  /**
   * @description Returns ACL for the server.
   * @param {number} nId - server id
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   */
  GetScVServerAcl ({
                     nId
                   }, connection) {
    let params = {
      nId
    }
    return this.baseRequest({
      command: `${className}.GetScVServerAcl`,
      data: params,
      connection
    })
               .then(body => this.deserialize(body, 'pAclParams'))
  }

  /**
   * @description Set ACL for virtual server.
   * @param {number} nId - virtual server id
   * @param {object} pAclParams - acl
   * @param {boolean} bCheckCurrentUserAce - if true then it checks that the user does not reduce rights for himself
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   */
  SetScVServerAcl ({
                     nId = 1,
                     pAclParams,
                     bCheckCurrentUserAce = false
                   }, connection) {
    var params = {
      nId,
      pAclParams,
      bCheckCurrentUserAce
    }
    return this.baseRequest({
      command: `${className}.SetScVServerAcl`,
      data: params,
      connection
    })
               .then(body => {
                 return this.deserialize(body, 'pAclParams')
               })
  }

  /**
   * @description Returns accessible functional areas.
   * @param lGroupId {number} - id of the group
   * @param dwAccessMask {number} - access mask
   * @param szwProduct {string} - product
   * @param szwVersion {string} - version of product
   * @param bInvert {boolean} - if true then access mask is inverted
   * @param connection
   * @returns {Promise.<TResult>} - array of functionality areas. Each element of the array is a string: "<product>|<version>|<functional area>".
   */
  GetAccessibleFuncAreas ({
    lGroupId,
    dwAccessMask,
    szwProduct,
    szwVersion,
    bInvert
  }, connection) {
    let params = {
      lGroupId,
      dwAccessMask,
      szwProduct,
      szwVersion,
      bInvert
    }
    return this.baseRequest({
                              command: `${className}.GetAccessibleFuncAreas`,
                              data: params,
                              connection
                            })
               .then(body => this.deserialize(body, 'pFuncAreasArray'))
  }

  /**
   * @description Returns unaccessible functional areas.
   * @param lGroupId {number} - id of the group
   * @param szwProduct {string} - product
   * @param szwVersion {string} - version of product
   * @param connection
   * @returns {Promise.<TResult>} - array of functionality areas. Each element of the array is a string: "<product>|<version>|<functional area>".
   */
  GetUnAccessibleFuncAreas ({
    lGroupId,
    szwProduct,
    szwVersion
  }, connection) {
    let params = {
      lGroupId,
      dwAccessMask: 1,
      szwProduct,
      szwVersion,
      bInvert: true
    }
    return this.GetAccessibleFuncAreas(params, connection)
               .then(res => {
                  return {result: res.result.map( item => item.split('|')[2] )}
               })
  }

  /**
   * @description Checks if current user session has access to the administration group.
   * @param lGroupId {number} - id of the group
   * @param dwAccessMask {number} - access mask
   * @param szwFuncArea {string} - functional area
   * @param szwProduct {string} - product
   * @param szwVersion {string} - version of product
   * @returns {Promise.<TResult>} {boolean} - true if the user has access else false
   */
  AccessCheckToAdmGroup ({
    lGroupId,
    dwAccessMask,
    szwFuncArea,
    szwProduct,
    szwVersion
  }, connection) {
let params = {
lGroupId,
dwAccessMask,
szwFuncArea,
szwProduct,
szwVersion
}
return this.baseRequest({
        command: `${className}.AccessCheckToAdmGroup`,
        data: params,
        connection
      })
.then(body => this.getResult(body))
}

  /**
   * @description Returns mapping functional area to tasks.
   * @param szwProduct {string} - product
   * @param szwVersion {string} - version of product
   * @param connection
   * @returns {Promise.<TResult>} - mapping of functional areas to tasks
   */
  GetMappingFuncAreaToTasks({
    szwProduct,
    szwVersion
  }, connection) {
    let params = {
      szwProduct,
      szwVersion
    }
    return this.baseRequest({
                              command: `${className}.GetMappingFuncAreaToTasks`,
                              data: params,
                              connection
                            })
               .then(body => this.getResult(body))
  }

  GetPolicyReadonlyNodes({
    lGroupId,
    szwProduct,
    szwVersion,
    szwSectionName,
    pPolicySection
  }, connection) {
    let params = {
      lGroupId,
      szwProduct,
      szwVersion,
      szwSectionName,
      pPolicySection
    }
    return this
      .baseRequest({
        command: `${className}.GetPolicyReadonlyNodes`,
        data: params,
        connection
      })
      .then(body => this.getResult(body))
  }
}
