const BaseOpenApiClient = require('../base-open-api-client')
const className = 'ListTags'

module.exports = class ListTags extends BaseOpenApiClient {
  /**
   * @description Add a new tag. Allows to add a new tag value that can be set for a list item
   * @param {string} tagsType - "HostsTags" || "InventoryTags" || "UmdmDeviceTags"
   * @param {string} szwTagValue - non-empty tag value of type
   * @param {object} pParams - reserved
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   */
  AddTag ({ tagsType, szwTagValue, pParams = null } = {}, connection) {
    return this.baseRequest({
      command: `${tagsType}.${className}.AddTag`,
      data: { szwTagValue, pParams },
      connection
    })
      .then(this.deserialize)
  }
  /**
   * @description Delete tags. Allows to delete multiple tag values in one method call
   * @param {string} tagsType - "HostsTags" || "InventoryTags" || "UmdmDeviceTags"
   * @param {Array<string>} pTagValue - tags that will be deleted
   * @param {object} pParams - reserved
   * @param {Connection} connection
   */
  DeleteTags2 ({ tagsType, pTagValue, pParams = null } = {}, connection) {
    return this.baseRequest({
      command: `${tagsType}.${className}.DeleteTags2`,
      data: { pTagValue, pParams },
      connection
    })
      .then(this.deserialize)
  }
  /**
   * @description Get all existing tag values. Retrieves all known tag values that can be set for a list item
   * @param {string} tagsType - "HostsTags" || "InventoryTags" || "UmdmDeviceTags"
   * @param {object} pParams - object with attributes:
   * "KLTAGS_INCLUDE_VS" optional (boolean) flag. true(default) - to include tags information from virtual servers, false - to exclude tags information from virtual servers
   * @param {Connection} connection
   * @returns {Promise.<TResult>}
   */
  GetAllTags ({ tagsType, pParams, deserialize }, connection) {
    return this.baseRequest({
      command: `${tagsType}.${className}.GetAllTags`,
      data: { pParams },
      connection
    })
      .then(deserialize ? this.deserialize : this.getResult)
  }
  /**
   * @description Get tag values for specified list items. Retrieves tag values for specified list items
   * @param {string} tagsType - "HostsTags" || "InventoryTags" || "UmdmDeviceTags"
   * @param {Array<any>} pIdArray - collection of list item identifiers . Value semantics depends on the list type (for HostsTags it is the host id and so on)
   * @param {object} pParams - reserved
   * @param {Connection} connection
   * @returns {Promise.<TResult>}
   */
  GetTags ({ tagsType, pIdArray, pParams = null }, connection) {
    return this.baseRequest({
      command: `${tagsType}.${className}.GetTags`,
      data: { pIdArray, pParams },
      connection
    }).then(this.getResult)
  }
  /**
   * @description Rename a tag. Allows to rename a tag
   * @param {string} tagsType - "HostsTags" || "InventoryTags" || "UmdmDeviceTags"
   * @param {string} szwOldTagValue - non-empty old tag value of type
   * @param {string} szwNewTagValue - non-empty new tag value of type
   * @param {object} pParams - reserved
   * @param {Connection} connection
   * @returns {Promise.<TResult>}
   */
  RenameTag ({ tagsType, szwOldTagValue, szwNewTagValue, pParams = null }, connection) {
    return this.baseRequest({
      command: `${tagsType}.${className}.RenameTag`,
      data: { szwOldTagValue, szwNewTagValue, pParams },
      connection
    }).then(this.getResult)
  }
  /**
   * @description Set tag values for specified list items. Assign tag values for specified list items
   * @param {string} tagsType - "HostsTags" || "InventoryTags" || "UmdmDeviceTags"
   * @param {Array<any>} pListItemsTags - collection of objects where each of them has the following structure:
   *  "KLTAGS_ITEM_ID" - list item identifier. Type depends on the list type
   *  "KLTAGS_TAGS" - (array) collection of (params) objects where each of them has the following structure:
   *    "KLTAGS_VALUE" - (wstring) value of a tag being set
   *    "KLTAGS_SET" - (boolean) flag. true - to set tag, false - to reset it
   * @param {object} pParams -

 object of the following structure:
   *  "KLTAGS_FULL_REPLACE" - optional (boolean) flag. true - to make a full replacement of previously set up tags, false(default) - to set up this tag and leave the old ones untouched
   * @param {Connection} connection
   * @returns {Promise}
   */
  SetTags ({ tagsType, pListItemsTags, pParams = null }, connection) {
    
    return this.baseRequest({
      command: `${tagsType}.${className}.SetTags`,
      data: { pListItemsTags: this.serialize(pListItemsTags), pParams },
      connection
    }).then(this.getResult)
  }
}
