const BaseOpenApiClient = require('../base-open-api-client')
const className = 'PolicyProfiles'

module.exports = class PolicyProfiles extends BaseOpenApiClient {
  /**
   * Returns array of all profiles for the specified policy
   * @param {number} nPolicy - policy identifier
   * @param {number} nRevision - policy revision id, zero means 'current policy'
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   * @constructor
   */
  EnumProfiles ({nPolicy, nRevision = 0} = {}, connection) {
    nPolicy = parseInt(nPolicy)
    nRevision = parseInt(nRevision)
    return this.baseRequest({
      command: `${className}.EnumProfiles`,
      data: {nPolicy, nRevision},
      connection
    })
      .then(this.deserialize)
  }

  /**
   * Returns profile data for the specified policy profile
   * @param {number} nPolicy - policy id
   * @param {number} nRevision - policy revision id, zero means 'current policy'
   * @param {string} szwName - profile name, a non-empty string, up to 100 unicode characters
   * @param {Connection} connection
   * @constructor
   */
  GetProfile ({nPolicy, nRevision = 0, szwName} = {}, connection) {
    return this.baseRequest({
      command: `${className}.GetProfile`,
      data: {nPolicy, nRevision, szwName},
      connection
    })
      .then(this.deserialize)
  }

  /**
   * Return identifier of opened SsContents, must be closed with SsContents::SS_Release
   * @param {number} nPolicy - policy id
   * @param {number} nRevision - policy revision id, zero means 'current policy'
   * @param {string} szwName - profile name, a non-empty string, up to 100 unicode characters
   * @param {number} nLifeTime -  timeout in milliseconds to keep this SsContents object alive, zero means 'default value'
   * @param {Connection} connection
   * @constructor
   */
  GetProfileSettings ({nPolicy, nRevision = 0, szwName, nLifeTime = 0} = {}, connection) {
    return this.baseRequest({
      command: `${className}.GetProfileSettings`,
      data: {nPolicy, nRevision, szwName, nLifeTime},
      connection
    })
      .then(this.deserialize)
  }

  /**
   * Create a new profile
   *
   * Creates a new profile with the specified name.
   * The KLSSPOL_PRF_EXPRESSION must contain a unique profile name otherwise an exception
   * KLSTD::STDE_OBJ_EXISTS is thrown.
   *
   * Actually the profile is saved only after the KLPRSSP::Contents::ApplyChanges method is called.
   * So, the KLPRSSP::Contents::ApplyChanges method may also throw an exception
   * if the profile is not unique
   *
   * @param {number} nPolicy - policy id
   * @param {string} szwName - profile name, a non-empty string, up to 100 unicode characters
   * @param {object} pAttrs - profile data, following attributes may be specified in the policy format
   * @param {number} nLifeTime - timeout in milliseconds to keep this SsContents object alive, zero means 'default value'
   * @param {Connection} connection
   * @returns {Promise.<TResult>}
   */
  async AddProfile ({nPolicy, szwName, pAttrs, nLifeTime = 0}, connection) {
    const result = await this.baseRequest({
      command: `${className}.AddProfile`,
      data: {
        nPolicy,
        szwName,
        pAttrs: this.serialize(pAttrs),
        nLifeTime
      },
      connection
    })
    return this.getResult(result)
  }

  /**
   * Delete profile
   *
   * Deletes the specified profile and all data associated.
   * Profile with KLSSPOL_PRF_PROTECTED set to true cannot be deleted.
   *
   * @param {number} nPolicy - policy id
   * @param {string} szwName - profile name, a non-empty string, up to 100 unicode characters
   * @param {Connection} connection
   * @returns {Promise.<TResult>}
   */
  DeleteProfile ({nPolicy, szwName}, connection) {
    return this.baseRequest({
      command: `${className}.DeleteProfile`,
      data: {nPolicy, szwName},
      connection
    }).then(this.getResult)
  }

  /**
   * Update attributes of an existing profile
   * Changes attributes of an existing profile.
   *
   * @param {number} nPolicy - policy id
   * @param {string} szwName - profile name, a non-empty string, up to 100 unicode characters
   * @param {object} pAttrsToUpdate - profile data, following attributes may be specified
   * @param {Connection} connection
   * @returns {Promise.<TResult>}
   */
  UpdateProfile ({nPolicy, szwName, pAttrsToUpdate}, connection) {
    return this.baseRequest({
      command: `${className}.UpdateProfile`,
      data: {nPolicy, szwName, pAttrsToUpdate},
      connection
    }).then(this.getResult)
  }

  /**
   * Acquire profile priority array
   * Returns array of profile names, the profile with lesser index has greater priority
   * @param {number} nPolicy - policy id
   * @param {number} nRevision - policy revision id, zero means 'current policy'
   * @param {Connection} connection
   * @returns {Promise}
   */
  GetPriorities ({nPolicy, nRevision = 0}, connection) {
    return this.baseRequest({
      command: `${className}.GetPriorities`,
      data: {nPolicy, nRevision},
      connection
    }).then(this.getResult)
  }

  PutPriorities ({nPolicy, pArrayOfNames}, connection) {
    return this.baseRequest({
      command: `${className}.PutPriorities`,
      data: {nPolicy, pArrayOfNames},
      connection
    }).then(this.getResult)
  }

  /**
   * Переименование профиля
   * @param {number} nPolicy - идентификатор политики
   * @param {string} szwExistingName - текущее имя пользователя
   * @param {string} szwNewName - новое имя пользователя
   * @constructor
   */
  RenameProfile ({nPolicy, szwExistingName, szwNewName}, connection) {
    return this.baseRequest({
      command: `${className}.RenameProfile`,
      data: {nPolicy, szwExistingName, szwNewName},
      connection
    }).then(this.getBody)
  }

  GetEffectivePolicyContents ({nPolicy, szwHostId, nLifeTime = 0}, connection) {
    return this.baseRequest({
      command: `${className}.GetEffectivePolicyContents`,
      data: {nPolicy, szwHostId, nLifeTime},
      connection
    }).then(this.getResult)
  }
}
