const BaseOpenAPIClient = require('../base-open-api-client')
const className = 'SecurityPolicy'

/**
 * Allows to manage users and permissions.
 */
module.exports = class SecurityPolicy extends BaseOpenAPIClient {
  /**
   * @description Acquire existing user properties.
   * @param {number} lUserId - user id
   * @param {number} lVsId - user id
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   */
  GetUsers ({ lUserId, lVsId = 0 }, connection) {
    return this.baseRequest({
      command: `${className}.GetUsers`,
      data: { lUserId, lVsId },
      connection
    })
      .then(this.deserialize)
  }

  /**
   * @description Add new user.
   * @param {object} pUser - user info, containing attributes "KLSPL_USER_*"
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   */
  AddUser ({ pUser }, connection) {
    return this.baseRequest({
      command: `${className}.AddUser`,
      data: {
        pUser
      },
      connection,
      tranformRequestBodyLogs: body => {
        const password = _.get(body, 'pUser.KLSPL_USER_PWD')
        if (password !== undefined) {
          body.pUser.KLSPL_USER_PWD = '🙈🙉🙊'
        }
        return body
      }
    })
      .then(this.getResult)
  }

  /**
   * @description Modify existing user properties.
   * @param {number} lUserId - user id
   * @param {object} pUser - user info, containing attributes "KLSPL_USER_*"
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   */
  UpdateUser ({ lUserId, pUser }, connection) {
    return this.baseRequest({
      command: `${className}.UpdateUser`,
      data: {
        lUserId,
        pUser
      },
      connection,
      tranformRequestBodyLogs: body => {
        const password = _.get(body, 'pUser.KLSPL_USER_PWD')
        if (password !== undefined) {
          body.pUser.KLSPL_USER_PWD = '🙈🙉🙊'
        }
        return body
      }
    })
      .then(this.getResult)
  }

  /**
   * @description Modify internal/external user of external group properties
   *
   * @param {long} llTrusteeId - Unique group/user ID
   * @param {object} pUserData - User/group data
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   */
  UpdateTrustee ({ llTrusteeId, pUserData }, connection) {
    return this.baseRequest({
      command: `${className}.UpdateTrustee`,
      data: {
        llTrusteeId,
        pUserData: this.serialize(pUserData)
      },
      connection
    })
      .then(this.getResult)
  }

  /**
   * @description Delete user.
   * @param {number} lUserId - user id
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   */
  DeleteUser ({lUserId}, connection) {
    return this.baseRequest({
      command: `${className}.DeleteUser`,
      data: {
        lUserId
      },
      connection
    })
      .then(this.getResult)
  }

  /**
   * @description Acquire current user id.
   * @param {number} lUserId - user id
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   */
  GetCurrentUserId (lUserId, connection) {
    return this.baseRequest({
      command: `${className}.GetCurrentUserId`,
      data: {
        lUserId
      },
      connection
    })
      .then(this.getBody)
  }

  /**
   * @description Acquire current user id.
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   */
  GetCurrentUserId2 (noParams, connection) {
    return this.baseRequest({
      command: `${className}.GetCurrentUserId2`,
      connection
    })
      .then(this.getBody)
  }

  /**
   * @description Modify a password of the existing user..
   * @param {number} lUserId - user id
   * @param {string} wstrOldPassword - MD5 hash of old user's password
   * @param {string} wstrNewPassword - MD5 hash of new user's password
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   */
  ChangeUserPassword ({ lUserId, wstrOldPassword, wstrNewPassword }, connection) {
    return this.baseRequest({
      command: `${className}.ChangeUserPassword`,
      data: {
        lUserId,
        wstrOldPassword,
        wstrNewPassword
      },
      connection
    })
      .then(this.getResult)
  }
}
