const baseOpenAPIClient = require('../base-open-api-client')
const className = 'ServerHierarchy'

module.exports = class ServerHierarchy extends baseOpenAPIClient {
  /**
   * Removes slave server registration record from the master server
   * @param {int} lServer - Slave server id
   */
  DelServer ({ lServer }, connection) {
    return this.baseRequest({
      command: `${className}.DelServer`,
      data: {
        lServer
      },
      connection
    }).then(this.getBody)
  }

  /**
   * Searches for slave servers meeting specified criteria. In filter expression may be also used c_szwSrchNow.
   * @param {string} wstrFilter - search filter
   * @param {Array<string>} pFieldsToReturn - search filter
   * @param {Array<string>} pFieldsToOrder - names of columns to sort by, same attributes as for pFieldsToReturn may be used
   * @param {object} pParams - additional params
   * @param {int} lMaxLifeTime - max lifetime of accessor (sec)
   * @returns {Promise.<TResult>}
   */
  FindSlaveServers ({ wstrFilter,  pFieldsToReturn = [], pFieldsToOrder = [], pParams = {}, lMaxLifeTime}, connection) {
    return this.baseRequest({
      command: `${className}.FindSlaveServers`,
      data: {
        wstrFilter,
        pFieldsToReturn,
        pFieldsToOrder,
        pParams,
        lMaxLifeTime
      },
      connection
    }).then(this.getBody)
  }

  /**
   * Enumerates slave servers for specified administration group and returns their attributes.
   * @param {int} nGroupId - group id, one may specify -1 to acquire slave servers from all groups
   * @returns {Promise.<array>} - each element is object containing attributes of slave server
   */
  GetChildServers ({ nGroupId }, connection) {
    return this.baseRequest({
      command: `${className}.GetChildServers`,
      data: {
        nGroupId
      },
      connection
    })
    .then(this.deserialize)
  }

  /**
   * Returns specified attributes of given slave server.
   * @param {int} lServer - Slave server id
   * @param {Array<string>} pFields - array of names of slave server attributes to return
   * @returns {Promise.<TResult>} - object containing specified attributes of slave server
   */
  GetServerInfo ({ lServer, pFields }, connection) {
    return this.baseRequest({
      command: `${className}.GetServerInfo`,
      data: {
        lServer,
        pFields
      },
      connection
    }).then(this.deserialize)
  }

  /**
   * Register new slave server for the specified group.
   * @param {string} wstrDisplName - slave server display name
   * @param {int} nGroupId - group id, for the slave server
   * @param {string} pCertificate - certificate binary data
   * @param {string} wstrNetAddress - server net address
   * @param {object} pAdditionalInfo - Additional information for the slave server
   * @returns {Promise.<int>} - slave server id
   */
  RegisterServer ({ wstrDisplName, nGroupId, pCertificate, wstrNetAddress, pAdditionalInfo  }, connection) {
    return this.baseRequest({
      command: `${className}.RegisterServer`,
      data: this.serialize({
        wstrDisplName,
        nGroupId,
        pCertificate,
        wstrNetAddress,
        pAdditionalInfo
      }),
      connection
    }).then(this.getResult)
  }

  /**
   * Modify specified attributes of given slave server
   * @param {int} lServer - Slave server id
   * @param {object} pInfo - container with slave server attributes
   * @returns {Promise.<TResult>}
   */
  UpdateServer ({ lServer, pInfo }, connection) {
    return this.baseRequest({
      command: `${className}.UpdateServer`,
      data: {
        lServer,
        pInfo
      },
      connection
    }).then(this.getBody)
  }
}
