const BaseOpenAPIClient = require('../base-open-api-client')
const className = 'VServers'

module.exports = class VServers extends BaseOpenAPIClient {
  /**
   * Registers new virtual server
   * @param {string} strDisplayName - virtual server display name, if display name is non-unique, it will be modified to become unique
   * @param {int} lParentGroup - virtual server parent group
   * @returns {Promise.<TResult>}
   */
  AddVServerInfo ({ strDisplayName, lParentGroup }, connection) {
    return this.baseRequest({
      command: `${className}.AddVServerInfo`,
      data: {
        strDisplayName,
        lParentGroup
      },
      connection
    })
    .then(this.deserialize)
  }

  /**
   * Unregisters specified Virtual Server
   * @param {int} lVServer - virtual server id
   * @returns {Promise.<string>} - id of asynchronous operation
   */
  DelVServer ({ lVServer }, connection) {
    return this.baseRequest({
      command: `${className}.DelVServer`,
      data: {
        lVServer
      },
      connection
    })
    .then(this.getBody)
  }

  /**
   * Returns ACL for the specified virtual server
   * @param {int} lVServer - virtual server id
   * @returns {Promise.<object>} - ACL
   */
  GetPermissions ({ lVServer }, connection) {
    return this.baseRequest({
      command: `${className}.GetPermissions`,
      data: {
        lVServer
      },
      connection
    })
    .then(this.deserialize)
  }

  /**
   * @description Acquire virtual servers for the specified group. Returns array of virtual servers for the specified group.
   * @param {number} lParentGroup - id of parent group, -1 means 'from all groups'
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   */
  GetVServers ({ lParentGroup = -1 }, connection) {
    const params = { lParentGroup }
    return this.baseRequest({
      command: `${className}.GetVServers`,
      data: params,
      connection
    })
    .then(this.deserialize)
  }
  /**
   * @description Acquire info on virtual server. Returns info about the specified virtual server
   * @param {number} lVServer - virtual server id
   * @param {number} pFields2Return - attributes "KLVSRV_*" to acquire
   * @param {Connection} connection
   * @return {Promise.<TResult>}
   */
  GetVServerInfo ({ lVServer, pFields2Return }, connection) {
    const params = { lVServer, pFields2Return }
    return this.baseRequest({
      command: `${className}.GetVServerInfo`,
      data: params,
      connection
    })
    .then(this.deserialize)
  }

  /**
   * Moves specified virtual server
   * @param {int} lVServer - virtual Server id
   * @param {int} lNewParentGroup - new group id
   * @returns {Promise.<{string} strActionGuid>} - id of asynchronous operation
   */
  MoveVServer ({ lVServer, lNewParentGroup }, connection) {
    return this.baseRequest({
      command: `${className}.MoveVServer`,
      data: {
        lVServer,
        lNewParentGroup
      },
      connection
    })
    .then(this.deserialize)
  }

  /**
   * Function recalls Network Agent certificate from the specified virtual server and closes active connections from such Network Agents
   * @param {int} lVServer - virtual server id
   */
  RecallCertAndCloseConnections ({ lVServer }, connection) {
    return this.baseRequest({
      command: `${className}.RecallCertAndCloseConnections`,
      data: {
        lVServer
      },
      connection
    })
    .then(this.deserialize)
  }

  /**
   * Sets ACL for the specified virtual server
   * @param {int} lVServer - virtual server id
   * @param {object} pPermissions - ACL
   * @param {boolean} bProtection - if true checks if the user does not denies access to the server to itself
   * @returns {Promise.<TResult>}
   */
  SetPermissions ({ lVServer, pPermissions, bProtection = true }, connection) {
    return this.baseRequest({
      command: `${className}.SetPermissions`,
      data: this.serialize({
        lVServer,
        pPermissions,
        bProtection
      }),
      connection
    })
    .then(this.deserialize)
  }

  /**
   * Modifies attributes of the specified virtual server
   * @param {int} lVServer - virtual server id
   * @param {object} pInfo - a container containing no-read-only attributes "KLVSRV_*" to change
   */
  UpdateVServerInfo ({ lVServer, pInfo }, connection) {
    return this.baseRequest({
      command: `${className}.UpdateVServerInfo`,
      data: {
        lVServer,
        pInfo
      },
      connection
    })
    .then(this.deserialize)
  }
}
